2021-09-28 22:55 に shinya11.k...@nttdata.com さんは書きました:
-----Original Message-----
From: bt21tanigaway <bt21taniga...@oss.nttdata.com>
Sent: Tuesday, September 28, 2021 5:06 PM
To: Fujii Masao <masao.fu...@oss.nttdata.com>;
pgsql-hackers@lists.postgresql.org
Subject: Re: (LOCK TABLE options) “ONLY” and “NOWAIT” are not yet
implemented

2021-09-28 17:03 に bt21tanigaway さんは書きました:
2021-09-28 16:36 に Fujii Masao さんは書きました:
On 2021/09/28 16:13, bt21tanigaway wrote:
Hi,

(LOCK TABLE options) “ONLY” and “NOWAIT” are not yet implemented in
tab-complete. I made a patch for these options.

Thanks for the patch!
The patch seems to forget to handle the tab-completion for "LOCK ONLY
<table-name> IN".

Thanks for your comment!
I attach a new patch fixed to this mail.

Regards,

Koyu Tanigawa

Sorry, I forgot to attach patch file.
"fix-tab-complete2.patch" is fixed!

Regards,

Koyu Tanigawa
Thank you for your patch.
I have two comments.

1. When I executed git apply, an error occured.
---
$ git apply ~/Downloads/fix-tab-complete2.patch
/home/penguin/Downloads/fix-tab-complete2.patch:14: indent with spaces.
    COMPLETE_WITH_SCHEMA_QUERY(Query_for_list_of_tables, " UNION
SELECT 'TABLE'" " UNION SELECT 'ONLY'");
warning: 1 line adds whitespace errors.
---

Thank you for your feedback.
I might have added whitespace when I was checking the patch file.
I attach a new patch to this mail.

2. The command "LOCK TABLE a, b;" can be executed, but tab-completion
doesn't work properly. Is it OK?
It's OK for now.
But it should be able to handle a case of multiple tables in the future.

Regards,

Koyu Tanigawa
diff --git a/src/bin/psql/tab-complete.c b/src/bin/psql/tab-complete.c
index 5cd5838668..eada1f453c 100644
--- a/src/bin/psql/tab-complete.c
+++ b/src/bin/psql/tab-complete.c
@@ -3599,39 +3599,52 @@ psql_completion(const char *text, int start, int end)
 		COMPLETE_WITH("(");
 
 /* LOCK */
 	/* Complete LOCK [TABLE] with a list of tables */
 	else if (Matches("LOCK"))
-		COMPLETE_WITH_SCHEMA_QUERY(Query_for_list_of_tables,
-								   " UNION SELECT 'TABLE'");
+		COMPLETE_WITH_SCHEMA_QUERY(Query_for_list_of_tables, " UNION SELECT 'TABLE'" " UNION SELECT 'ONLY'");
+
 	else if (Matches("LOCK", "TABLE"))
-		COMPLETE_WITH_SCHEMA_QUERY(Query_for_list_of_tables, "");
+		COMPLETE_WITH_SCHEMA_QUERY(Query_for_list_of_tables, " UNION SELECT 'ONLY'");
 
+	else if (Matches("LOCK", "TABLE", "ONLY") || Matches("LOCK", "ONLY"))
+		COMPLETE_WITH_SCHEMA_QUERY(Query_for_list_of_tables, "");
 	/* For the following, handle the case of a single table only for now */
-
-	/* Complete LOCK [TABLE] <table> with "IN" */
-	else if (Matches("LOCK", MatchAnyExcept("TABLE")) ||
-			 Matches("LOCK", "TABLE", MatchAny))
-		COMPLETE_WITH("IN");
-
-	/* Complete LOCK [TABLE] <table> IN with a lock mode */
+	
+	/* Complete LOCK [TABLE] [ONLY] <table> with "IN" or "NOWAIT" */
+	else if (Matches("LOCK", MatchAnyExcept("TABLE|ONLY")) ||
+			 Matches("LOCK", "TABLE", MatchAnyExcept("ONLY")) ||
+			 Matches("LOCK", "ONLY", MatchAny) ||
+			 Matches("LOCK", "TABLE", "ONLY", MatchAny))
+		COMPLETE_WITH("IN", "NOWAIT");
+
+	/* Complete LOCK [TABLE] [ONLY] <table> IN with a lock mode */
 	else if (Matches("LOCK", MatchAny, "IN") ||
-			 Matches("LOCK", "TABLE", MatchAny, "IN"))
+			 Matches("LOCK", "TABLE", MatchAny, "IN") ||
+			 Matches("LOCK", "ONLY", MatchAny, "IN") ||
+			 Matches("LOCK", "TABLE", "ONLY", MatchAny, "IN"))
 		COMPLETE_WITH("ACCESS SHARE MODE",
 					  "ROW SHARE MODE", "ROW EXCLUSIVE MODE",
 					  "SHARE UPDATE EXCLUSIVE MODE", "SHARE MODE",
 					  "SHARE ROW EXCLUSIVE MODE",
 					  "EXCLUSIVE MODE", "ACCESS EXCLUSIVE MODE");
 
 	/* Complete LOCK [TABLE] <table> IN ACCESS|ROW with rest of lock mode */
 	else if (Matches("LOCK", MatchAny, "IN", "ACCESS|ROW") ||
-			 Matches("LOCK", "TABLE", MatchAny, "IN", "ACCESS|ROW"))
+			 Matches("LOCK", "TABLE", MatchAny, "IN", "ACCESS|ROW") ||
+			 Matches("LOCK", "ONLY", MatchAny, "IN", "ACCESS|ROW") ||
+			 Matches("LOCK", "TABLE", "ONLY", MatchAny, "IN", "ACCESS|ROW"))
 		COMPLETE_WITH("EXCLUSIVE MODE", "SHARE MODE");
 
 	/* Complete LOCK [TABLE] <table> IN SHARE with rest of lock mode */
 	else if (Matches("LOCK", MatchAny, "IN", "SHARE") ||
-			 Matches("LOCK", "TABLE", MatchAny, "IN", "SHARE"))
-		COMPLETE_WITH("MODE", "ROW EXCLUSIVE MODE",
-					  "UPDATE EXCLUSIVE MODE");
+			 Matches("LOCK", "TABLE", MatchAny, "IN", "SHARE") ||
+			 Matches("LOCK", "ONLY", MatchAny, "IN", "SHARE") ||
+			 Matches("LOCK", "TABLE", "ONLY", MatchAny, "IN", "SHARE"))
+		COMPLETE_WITH("MODE", "ROW EXCLUSIVE MODE", "UPDATE EXCLUSIVE MODE");
+
+  /* Complete LOCK [TABLE] [ONLY] <table> [IN lockmode MODE] with "NOWAIT"*/
+	else if (HeadMatches("LOCK") && TailMatches("MODE"))
+		COMPLETE_WITH("NOWAIT");
 
 /* NOTIFY --- can be inside EXPLAIN, RULE, etc */
 	else if (TailMatches("NOTIFY"))
@@ -3599,39 +3599,52 @@ psql_completion(const char *text, int start, int end)
 
 /* LOCK */
-	/* Complete LOCK [TABLE] with a list of tables */
+	/* Complete LOCK [TABLE] [ONLY] with a list of tables */
 	else if (Matches("LOCK"))
@@ -3599,39 +3599,52 @@ psql_completion(const char *text, int start, int end)
 					  "EXCLUSIVE MODE", "ACCESS EXCLUSIVE MODE");
 
-	/* Complete LOCK [TABLE] <table> IN ACCESS|ROW with rest of lock mode */
+	/* Complete LOCK [TABLE][ONLY] <table> IN ACCESS|ROW with rest of lock mode */
 	else if (Matches("LOCK", MatchAny, "IN", "ACCESS|ROW") ||
@@ -3599,39 +3599,52 @@ psql_completion(const char *text, int start, int end)
 		COMPLETE_WITH("EXCLUSIVE MODE", "SHARE MODE");
 
-	/* Complete LOCK [TABLE] <table> IN SHARE with rest of lock mode */
+	/* Complete LOCK [TABLE] [ONLY] <table> IN SHARE with rest of lock mode */
 	else if (Matches("LOCK", MatchAny, "IN", "SHARE") ||

Reply via email to